/*
 * IntrospectionPerformanceTest - Tests introspection on JavaBeans
 * Copyright (C) 2007 Christian Schenk
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */
package org.christianschenk.beanintrospect;

import org.christianschenk.beanintrospect.beans.A;
import org.christianschenk.beanintrospect.beans.B;
import org.christianschenk.beanintrospect.beans.C;
import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.fail;

public class BeanIntrospectorTest {

	private BeanIntrospector beanIntrospector;

	@Before
	public void setUp() {
		this.beanIntrospector = new BeanIntrospector();
	}

	@Test
	public void copy() {
		final A a1 = new A(1, 2);
		final A a2 = new A();
		this.beanIntrospector.fill(a1, a2);
		assertEquals(a1.getI(), a2.getI());
		assertEquals(a1.getJ(), a2.getJ());
	}

	@Test
	public void fillSmallerWithBiggerBean() {
		final A a = new A();
		final B b = new B(1, 2);
		this.beanIntrospector.fill(a, b);
		assertEquals(a.getI(), b.getI());
		assertEquals(a.getJ(), b.getJ());
	}

	@Test
	public void fillBiggerWithSmallerBean() {
		final A a = new A(1, 2);
		final B b = new B();

		try {
			this.beanIntrospector.fill(b, a);
			fail("Should throw exception");
		} catch (final Exception ex) {
		}

		// method name should ignore upper and lower case
		this.beanIntrospector.ignoreMethodName(B.class, "StR");
		this.beanIntrospector.fill(b, a);
		assertEquals(a.getI(), b.getI());
		assertEquals(a.getJ(), b.getJ());
		assertNull(b.getStr());
	}

	@Test
	public void mappingMethodNames() {
		final B b = new B(1, 2, "henner");
		final C c = new C();
		// method name mapping should ignore upper and lower case
		this.beanIntrospector.addMethodNameMapping(B.class, "StR", C.class, "hURz");
		this.beanIntrospector.fill(c, b);
		assertEquals(b.getStr(), c.getHurz());

		try {
			this.beanIntrospector.fill(b, c);
			fail("Should throw exception");
		} catch (final Exception ex) {
		}

		this.beanIntrospector.ignoreMethodName(B.class, "i");
		this.beanIntrospector.ignoreMethodName(B.class, "j");
		this.beanIntrospector.fill(b, c);
		assertEquals(b.getStr(), c.getHurz());
	}
}